Module attacks

Module attacks 

Chess board attack generation kernels using bitboards and SWAR.

Functions

bishop_attacks
Generate Bishop attacks for a square given occupancy using the Magic bitboard backend.
queen_attacks
Generate Queen attacks for a square given occupancy using the Magic bitboard backend.
rook_attacks
Generate Rook attacks for a square given occupancy using the Magic bitboard backend.