Generic runtime-dispatch in-place scale kernel.
scale_in_place<T>(data, scalar) broadcasts scalar to all SIMD lanes
then multiplies each chunk of data by it, covering the scalar tail
element-by-element.
Generic runtime-dispatch in-place scale kernel.
scale_in_place<T>(data, scalar) broadcasts scalar to all SIMD lanes
then multiplies each chunk of data by it, covering the scalar tail
element-by-element.