pub struct Aligned<const A: usize>;Expand description
A static alignment guarantee of A bytes.
Guaranteed at compile time to represent a power-of-two alignment boundary.
If A is not a power of two, it will fail to compile due to a const assertion check.
Examples
use hermes_simd_core::align::{Aligned, Alignment};
assert_eq!(Aligned::<32>::ALIGNMENT, Some(32));Trait Implementations
impl<const A: usize> Copy for Aligned<A>
impl<const A: usize> Eq for Aligned<A>
impl<const A: usize> StructuralPartialEq for Aligned<A>