#[repr(C, align(64))]pub struct AmxConfig {
pub palette_id: u8,
pub start_row: u8,
pub reserved: [u8; 14],
pub cols_b: [u16; 8],
pub reserved2: [u8; 16],
pub rows: [u8; 8],
pub reserved3: [u8; 8],
}Expand description
64-byte AMX tile configuration structure (TILECFG).
Fields
palette_id: u8Palette ID (0 = init, 1 = Palette 1)
start_row: u8Start row for recovery after interrupt / context switch
reserved: [u8; 14]Reserved bytes
cols_b: [u16; 8]Column widths in bytes for each of the 8 tiles
reserved2: [u8; 16]Reserved bytes
rows: [u8; 8]Number of rows for each of the 8 tiles
reserved3: [u8; 8]Reserved bytes
Implementations
Sourceimpl AmxConfig
impl AmxConfig
Sourcepub fn new_uniform(r: u8, c_bytes: u16) -> AmxConfig
pub fn new_uniform(r: u8, c_bytes: u16) -> AmxConfig
Creates a palette 1 configuration where all tiles have the specified rows and byte columns.