Packed4Slice

Struct Packed4Slice 

pub struct Packed4Slice<'a, T>
where T: Packable4,
{ /* private fields */ }
A read-only view over a packed slice of 4-bit values, stored 2 per byte.

Implementations

Source

impl<'a, T> Packed4Slice<'a, T>
where T: Packable4,

pub fn new(data: &'a [u8], len: usize) -> Option<Packed4Slice<'a, T>>

Create a new Packed4Slice from packed bytes and logical length. Returns None if the backing buffer is too small for the requested length.

pub fn len(&self) -> usize

Returns the logical length (number of elements).

pub fn as_packed_slice(&self) -> &'a [u8]

Returns the underlying packed byte slice.

pub fn is_empty(&self) -> bool

Returns true if empty.

pub fn get(&self, index: usize) -> Option<T>

Get an element at the given logical index.

pub fn sub_slice(self, range: Range<usize>) -> Option<Packed4Slice<'a, T>>

Attempts to create a zero-copy sub-slice of the packed slice.

This is only possible without allocation if the range starts at an even logical index (aligned to byte boundaries). If the range starts at an odd index, returns None.

Source

impl<'a, T> Packed4Slice<'a, T>
where T: Packable4,

pub fn unpack(&self, dest: &mut [<T as Packable4>::Unpacked])

Unpack all elements into a destination slice.

Source

impl<'a> Packed4Slice<'a, Bf4>

pub fn unpack_to_bf16(&self, dest: &mut [Bf16])

Unpack all elements into a destination slice of Bf16.

Source

impl<'a> Packed4Slice<'a, F4>

pub fn unpack_to_f32(&self, dest: &mut [F32])

Unpack all elements into a destination slice of F32.