Packed4SliceMut

Struct Packed4SliceMut 

pub struct Packed4SliceMut<'a, T>
where T: Packable4,
{ /* private fields */ }
A mutable view over a packed slice of 4-bit values, stored 2 per byte.

Implementations

Source

impl<'a, T> Packed4SliceMut<'a, T>
where T: Packable4,

pub fn new(data: &'a mut [u8], len: usize) -> Option<Packed4SliceMut<'a, T>>

Create a new Packed4SliceMut from packed bytes and logical length. Returns None if the backing buffer is too small for the requested length.

pub fn len(&self) -> usize

Returns the logical length (number of elements).

pub fn as_packed_slice(&self) -> &[u8]

Returns the underlying packed byte slice.

pub fn as_packed_slice_mut(&mut self) -> &mut [u8]

Returns the underlying mutable packed byte slice.

pub fn as_borrowed(&self) -> Packed4Slice<'_, T>

Return a borrowed read-only view of this mutable packed slice.

pub fn get(&self, index: usize) -> Option<T>

Get an element at the given logical index.

pub fn is_empty(&self) -> bool

Returns true if empty.

pub fn set(&mut self, index: usize, val: T)

Set an element at the given logical index.

pub fn sub_slice_mut( self, range: Range<usize>, ) -> Option<Packed4SliceMut<'a, T>>

Attempts to create a zero-copy mutable sub-slice of the packed slice.

This is only possible without allocation if the range starts at an even logical index (aligned to byte boundaries). If the range starts at an odd index, returns None.