SwarUtils

Struct SwarUtils 

pub struct SwarUtils;
Pure SWAR bitwise utility primitives.

Implementations

Source

impl SwarUtils

pub fn popcount(x: u64) -> u32

Count set bits in a 64-bit word using SWAR parallel popcount.

pub fn bit_scan_forward(x: u64) -> u32

Find the least significant set bit (bit index 0-63) using SWAR trailing zero count.

pub fn bit_scan_reverse(x: u64) -> u32

Find the most significant set bit (bit index 0-63) using SWAR leading zero count.

pub fn parallel_prefix_shift<F>(x: u64, shift_fn: F) -> u64
where F: Fn(u64, usize) -> u64,

Computes parallel prefix shift (Kogge-Stone style prefix scan) on a 64-bit mask.

pub fn isolate_lsb(x: u64) -> u64

Isolates the least significant set bit (all other bits set to zero).

pub fn clear_lsb(x: u64) -> u64

Clears the least significant set bit.

pub fn isolate_msb(x: u64) -> u64

Isolates the most significant set bit (all other bits set to zero).

pub fn popcount_8(x: u64) -> u64

Computes parallel popcount for each of the 8-bit fields in a 64-bit word.

pub fn popcount_16(x: u64) -> u64

Computes parallel popcount for each of the 16-bit fields in a 64-bit word.

pub fn popcount_32(x: u64) -> u64

Computes parallel popcount for each of the 32-bit fields in a 64-bit word.

pub fn blsmsk(x: u64) -> u64

Bit isolation mask: sets all bits from the lowest set bit to bit 0.

pub fn pext(val: u64, mask: u64) -> u64

SWAR implementation of parallel bit extract (pext). Extracts bits from val at positions specified by mask and packs them contiguously at the LSB.

pub fn pdep(val: u64, mask: u64) -> u64

SWAR implementation of parallel bit deposit (pdep). Deposits contiguous low bits from val to positions specified by mask.