TileView

Struct TileView 

pub struct TileView<'a, T, Backend, Arch, const ROWS: usize, const COLS: usize, Align = Unaligned, Ref = &'a [T]>
where T: NumericElement, Align: Alignment, Ref: 'a,
{ /* private fields */ }
A 2D matrix tile view, parameterized by dimensions, alignment, execution mode, and backing reference type.

Under the hood, this represents a 2D tile of shape ROWS x COLS with a specified row stride. It is represented as #[repr(C)] to guarantee layout compatibility.

Implementations

Source

impl<'a, T, Backend, Arch, const ROWS: usize, const COLS: usize, Align> TileView<'a, T, Backend, Arch, ROWS, COLS, Align>
where T: NumericElement, Arch: SimdArch, Align: Alignment,

pub fn new( data: &'a [T], stride: usize, ) -> Option<TileView<'a, T, Backend, Arch, ROWS, COLS, Align>>

Create a new read-only TileView after verifying bounds and alignment invariants. Returns None if the input slice is too small or if alignment constraints are not met.

Source

impl<'a, T, Backend, Arch, const ROWS: usize, const COLS: usize, Align> TileView<'a, T, Backend, Arch, ROWS, COLS, Align, &'a mut [T]>
where T: NumericElement, Arch: SimdArch, Align: Alignment,

pub fn new_mut( data: &'a mut [T], stride: usize, ) -> Option<TileView<'a, T, Backend, Arch, ROWS, COLS, Align, &'a mut [T]>>

Create a new mutable TileView after verifying bounds and alignment invariants. Returns None if the input slice is too small or if alignment constraints are not met.

pub fn as_mut_ptr(&mut self) -> *mut T

Access the underlying raw mutable pointer.

Source

impl<'a, T, Backend, Arch, const ROWS: usize, const COLS: usize, Align, Ref> TileView<'a, T, Backend, Arch, ROWS, COLS, Align, Ref>
where T: NumericElement, Align: Alignment, Ref: 'a,

pub fn as_ptr(&self) -> *const T

Access the underlying raw pointer.

pub fn stride(&self) -> usize

Returns the row stride of this tile view.

pub fn rows(&self) -> usize

Returns the number of rows.

pub fn cols(&self) -> usize

Returns the number of columns.