Module target

Module target 

Explicit runtime target tokens and forced dispatch helpers. Explicit SIMD target tokens and forced view dispatch helpers.

Enums

TargetId
Runtime-selectable SIMD target token for tests and benchmark harnesses.

Functions

dispatch_view_mut_to
Dispatches a mutable slice into an explicitly requested target.
dispatch_view_to
Dispatches a shared slice into an explicitly requested target.