Chess board attack generation kernels using bitboards and SWAR.
Functions
- bishop_
attacks - Generate Bishop attacks for a square given occupancy using the Magic bitboard backend.
- queen_
attacks - Generate Queen attacks for a square given occupancy using the Magic bitboard backend.
- rook_
attacks - Generate Rook attacks for a square given occupancy using the Magic bitboard backend.