pub struct SveArch;Expand description
AArch64 SVE architecture ZST marker.
The current stable implementation uses the same real lane-emulated kernel
family as other non-native (architecture, scalar) pairs. Native SVE
intrinsics can replace this implementation once stable Rust exposes the
required scalable-vector types.