Explicit runtime target tokens and forced dispatch helpers. Explicit SIMD target tokens and forced view dispatch helpers.
Enums
- Target
Id - Runtime-selectable SIMD target token for tests and benchmark harnesses.
Functions
- dispatch_
view_ mut_ to - Dispatches a mutable slice into an explicitly requested target.
- dispatch_
view_ to - Dispatches a shared slice into an explicitly requested target.