pub trait SparseFormat:
Sealed
+ Send
+ Sync
+ 'static {
type Storage<'a, T: 'a>: SparseShape;
const NAME: &'static str;
}Expand description
Marker trait for sparse matrix storage formats.
Sealed to prevent external format implementations. The GAT Storage<'a, T>
maps this format marker to its concrete data struct, eliminating the need
for an internal enum discriminant.
Required Associated Constants
Required Associated Types
Sourcetype Storage<'a, T: 'a>: SparseShape
type Storage<'a, T: 'a>: SparseShape
The concrete data struct type for this format, parameterized by
lifetime 'a and element type T.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".